#include <d3dx8.h>
#include <mmsystem.h>
#include <math.h>

#define FULLSCREEN 0
#define PI2 6.2831853071795864f
#define PI 3.14159265358979328f

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
class D3DObject
{
public:
	D3DObject()
	{
		g_pMesh          = NULL; // Our mesh object in sysmem
		g_pMeshMaterials = NULL; // Materials for our mesh
		g_pMeshTextures  = NULL; // Textures for our mesh
		g_dwNumMaterials = 0L;   // Number of mesh materials
		UseFlag=0;
	}

	LPD3DXMESH              g_pMesh          ; // Our mesh object in sysmem
	D3DMATERIAL8*           g_pMeshMaterials ; // Materials for our mesh
	LPDIRECT3DTEXTURE8*     g_pMeshTextures  ; // Textures for our mesh
	DWORD                   g_dwNumMaterials ;   // Number of mesh materials
	LPDIRECT3D8             *g_pD3D          ; // Used to create the D3DDevice
	LPDIRECT3DDEVICE8       *g_pd3dDevice     ; // Our rendering device
	HRESULT InitGeometry(char* ModelName);
	float x,y,z;
	float rx,ry,rz;
	int UseFlag;
	void DeleteObj()
	{
	   if( g_pMeshMaterials != NULL ) 
			delete[] g_pMeshMaterials;
		g_pMeshMaterials=NULL;
		if( g_pMeshTextures )
		{
			for( DWORD i = 0; i < g_dwNumMaterials; i++ )
			{
				if( g_pMeshTextures[i] )
					g_pMeshTextures[i]->Release();
			}
			delete[] g_pMeshTextures;
			g_pMeshTextures=NULL;
		}
		if( g_pMesh != NULL )
			g_pMesh->Release();
		g_pMesh=NULL;
	}
};
#define MAX_OBJECT 1000
class D3D
{
	LPDIRECT3D8             g_pD3D          ; // Used to create the D3DDevice
	LPDIRECT3DDEVICE8       g_pd3dDevice     ; // Our rendering device

	D3DObject Obj[MAX_OBJECT];
//	int MaxObj;
	HWND m_hWnd;
	void Error(LPSTR m);
public:
	char DebugString[256];
	D3D()
	{
		g_pD3D           = NULL; // Used to create the D3DDevice
		g_pd3dDevice     = NULL; // Our rendering device

//		MaxObj=0;
		SetCameraPos(0,0,-2,0,0,0);
		DebugString[0]=0;
	}
	/*
	HRESULT InitGeometry(char* ModelName)
	{
		Obj[MaxObj].g_pD3D=&g_pD3D;
		Obj[MaxObj].g_pd3dDevice=&g_pd3dDevice;
		int r=Obj[MaxObj].InitGeometry(ModelName);
		if(r==S_OK)
			MaxObj++;
		return r;
	}*/
	HRESULT InitD3D( HWND hWnd );
	void InitLight();
	VOID Cleanup();
	VOID Render();
	VOID SetupMatrices(float x,float y,float z,float rx=0.0f,float ry=0.0f,float rz=0.0f);
	void DrawObject(int num);
	int LoadObject(int num,LPSTR name)
	{
		if(num<0 || num>=MAX_OBJECT)
			return FALSE;
		Obj[num].DeleteObj();
		Obj[num].g_pD3D=&g_pD3D;
		Obj[num].g_pd3dDevice=&g_pd3dDevice;
		int r=Obj[num].InitGeometry(name);
		if(r==S_OK){
			Obj[num].UseFlag=1;
			return TRUE;
		}else{
			return FALSE;
		}
	}
	void SetObjectPos(int num,float x,float y,float z,float rx=0.0f,float ry=0.0f,float rz=0.0f)
	{
		if(num>=0 && num<MAX_OBJECT){
			Obj[num].x=x;
			Obj[num].y=y;
			Obj[num].z=z;
			Obj[num].rx=rx;
			Obj[num].ry=ry;
			Obj[num].rz=rz;
		}
	}
	float CameraX,CameraY,CameraZ;//J̈ʒu
	float CameraVX,CameraVY,CameraVZ;//J̌Ăꏊ
	void SetCameraPos(float vx,float vy,float vz,float x=0.0f,float y=0.0f,float z=0.0f)
	{
		CameraX=x;
		CameraY=y;
		CameraZ=z;
		CameraVX=vx;
		CameraVY=vy;
		CameraVZ=vz;
	}
};
